% Example 2: Recover sparse signal via random matrix and l-1 minimization
% This script demonstrates recovering sparese signal using random
% projection matrices.
%
% Matlab toolbox: cvx
% Example due to E. Candes.
% 
% Allen Y. Yang. Feb. 2008, UC Berkeley

clear;
close all;

n = 512;    % size of signal
S = 28;     % Sparsity

support = randsample(n,S);
x0 = zeros(n,1); x0(support) = randn(S,1);
figure(1); hold on
plot(1:n,x0,'b*');

% Sampling
m = 128;
A = randn(m,n); % Random matrix

% Normalize A matrix
vectorNorms = A*A.';
vectorNorms = sqrt(diag(vectorNorms));
vectorNorms = repmat(vectorNorms, [1,n]);
A = A./vectorNorms;

% Project
y = A*x0;
figure(2); 
plot(y);

% l-1 minimization
cvx_begin
    variable x(n);
    minimize(norm(x,1));
    subject to
        A*x == y;
cvx_end
figure(1);
plot(1:n,x,'ro');