% Example 4: Recover noisy sparse signal via l-1 minimization
% This script demonstrates a stable solution to handle the added l-2 noise
% term by changing the optimization condition
%
% Matlab toolbox: cvx
% 
% Allen Y. Yang. Feb. 2008, UC Berkeley

clear;
close all;

n = 512;    % size of signal
S = 28;     % Sparsity

support = randsample(n,S);
x0 = zeros(n,1); x0(support) = randn(S,1);
figure(1); hold on
plot(1:n,x0,'b*');

% Sampling
m = 64;
k=0:(n-1); t=0:(n-1);
F = exp(-j*2*pi*(k'*t)/n)/sqrt(n);  % Fourier matrix
freq = randsample(n,m);
A = [real(F(freq,:)); imag(F(freq,:))];

% Noisy projection
noise_bound = (1e-2)*rand();
noise = rand(size(A,1),1); noise = noise/norm(noise);
y = A*x0 + noise_bound*noise;
figure(2); 
plot(y);

% l-1 minimization
cvx_begin
    variable x(n);
    minimize(norm(x,1));
    subject to
        norm(A*x - y)<noise_bound;  % Explicitly model the added noise
cvx_end
figure(1);
plot(1:n,x,'ro');